import glob
import os
import re
import string


import vim


def catFile(filename):
    """ assigns a local variable retval to the contents of a file """
    try:
        fp = open(filename)
        lines = fp.read()
        fp.close()
    except FileNotFoundError:
        lines = ''

    # escape double quotes and backslashes before quoting the string so
    # everything passes throught.
    vim.command("""let retval = "%s" """ % re.sub(r'"|\\', r'\\\g<0>', lines))
    return lines


def isPresentInFile(regexp, filename):
    """ check if regexp is present in the file """
    try:
        fp = open(filename)
        fcontents = fp.read()
        fp.close()
        if re.search(regexp, fcontents):
            vim.command('let retval = 1')
            return 1
        else:
            vim.command('let retval = 0')
            return None
    except FileNotFoundError:
        vim.command('let retval = 0')
        return None


def deleteFile(filepattern):
    """ deletes a file if present

       If the file does not exist, check if its a filepattern rather than a
       filename. If its a pattern, then deletes all files matching the pattern.
    """
    try:
        if os.path.isfile(filepattern):
            os.remove(filepattern)
        else:
            if glob.glob(filepattern):
                for filename in glob.glob(filepattern):
                    os.remove(filename)
            else:
                vim.command('let retval = -1')
    except:
        vim.command('let retval = -1')

# vim:ff=unix:noet:ts=4:sw=4:nowrap



#!/usr/bin/env python

# Part of Latex-Suite
#
# Copyright: Srinath Avadhanula
# Description:
#   This file implements a simple outline creation for latex documents.

import re
import os
import sys
if sys.version_info <= (3, 0):
    from StringIO import StringIO
else:
    from io import StringIO


def getFileContents(fname):
    if type(fname) is not str:
        fname = fname.group(3)

    # If neither the file or file.tex exists, then we just give up.
    if not os.path.isfile(fname):
        if os.path.isfile(fname + '.tex'):
            fname += '.tex'
        else:
            return ''

    try:
        # This longish thing is to make sure that all files are converted into
        # \n seperated lines.
        contents = '\n'.join(open(fname).read().splitlines())
    except IOError:
        return ''

    # TODO what are all the ways in which a tex file can include another?
    pat = re.compile(r'^\s*\\(@?)(include|input){(.*?)}', re.M)
    contents = re.sub(pat, getFileContents, contents)

    return ('%%==== FILENAME: %s' % fname) + '\n' + contents


def stripComments(contents):
    # remove all comments except those of the form
    # %%==== FILENAME: <filename.tex>
    # BUG: This comment right after a new paragraph is not recognized:
    # foo\\%comment
    uncomm = [re.sub('(?<!\\\\)%(?!==== FILENAME: ).*', '', line)
              for line in contents.splitlines()]
    # also remove all only-whitespace lines.
    nonempty = [line for line in uncomm if line.strip()]

    return nonempty


def addFileNameAndNumber(lines):
    filename = ''
    retval = ''
    for line in lines:
        if re.match('%==== FILENAME: ', line):
            filename = line.split('%==== FILENAME: ')[1]
        else:
            retval += '<%s>%s\n' % (filename, line)

    return retval


def getSectionLabels_Root(lineinfo, section_prefix, label_prefix):
    prev_txt = ''
    inside_env = 0
    prev_env = ''
    outstr = StringIO('')
    pres_depth = len(section_prefix)
    indent = ' ' * (2*pres_depth + 2)

    #print '+getSectionLabels_Root: lineinfo = [%s]' % lineinfo
    for line in lineinfo.splitlines():
        if not line:
            continue

        # throw away leading white-space
        m = re.search('<(.*?)>(.*)', line)

        fname = m.group(1)
        line = m.group(2).lstrip()

        # we found a label!
        m = re.search(r'\\(?:nl)?label{(%s.*?)}' % label_prefix, line)
        if m:
            # Get the corresponding label
            label = m.group(1)

            # add the current line (except the \label command) to the text
            # which will be displayed below this label
            prev_txt += re.search(r'(^.*?)\\(?:nl)?label{', line).group(1)

            # for the figure environment however, just display the caption.
            # instead of everything since the \begin command.
            if prev_env == 'figure':
                cm = re.search(r'\\caption(\[.*?\]\s*)?{(.*?)}', prev_txt)
                if cm:
                    prev_txt = cm.group(2)

            # print a nice formatted text entry like so
            #
            # >        eqn:label
            # :          e^{i\pi} + 1 = 0
            #
            # Use the current "section depth" for the leading indentation.
            outstr.write('>%s%s\t\t<%s>\n' % (indent, label, fname))
            outstr.write(':%s  %s\n'       % (indent, prev_txt))

            prev_txt = ''

        # If we just encoutered the start or end of an environment or a
        # label, then do not remember this line.
        # NOTE: This assumes that there is no equation text on the same
        # line as the \begin or \end command. The text on the same line as
        # the \label was already handled.
        if re.search(r'\\begin{(equation|align|figure)', line):
            prev_txt = ''
            prev_env = re.search(r'\\begin{(.*?)}', line).group(1)
            inside_env = 1

        elif re.search(r'\\(?:nl)?label', line):
            prev_txt = ''

        elif re.search(r'\\end{(equation|align|figure)', line):
            inside_env = 0
            prev_env = ''

        else:
            # If we are inside an environment, then the text displayed with
            # the label is the complete text within the environment,
            # otherwise its just the previous line.
            if inside_env:
                prev_txt += line
            else:
                prev_txt = line

    return outstr.getvalue()


def getSectionLabels(lineinfo,
                     sectypes=['chapter', 'section',
                               'subsection', 'subsubsection'],
                     section_prefix='', label_prefix=''):

    if not sectypes:
        return getSectionLabels_Root(lineinfo, section_prefix, label_prefix)

    ##print 'sectypes[0] = %s, section_prefix = [%s], lineinfo = [%s]' % (
    ##        sectypes[0], section_prefix, lineinfo)

    sections = re.split(r'(<.*?>\\%s{.*})' % sectypes[0], lineinfo)

    # there will 1+2n sections, the first containing the "preamble" and the
    # others containing the child sections as paris of [section_name,
    # section_text]

    rettext = getSectionLabels(
        sections[0], sectypes[1:], section_prefix, label_prefix)

    for i in range(1, len(sections), 2):
        sec_num = (i + 1) / 2
        section_name = re.search(
            r'\\%s{(.*?)}' % sectypes[0], sections[i]).group(1)
        section_label_text = getSectionLabels(
            sections[i] + sections[i + 1], sectypes[1:],
            section_prefix + ('%d.' % sec_num), label_prefix)

        if section_label_text:
            sec_heading = 2 * ' ' * len(section_prefix) + section_prefix
            sec_heading += '%d. %s' % (sec_num, section_name)
            sec_heading += '<<<%d\n' % (len(section_prefix) / 2 + 1)

            rettext += sec_heading + section_label_text

    return rettext


def main(fname, label_prefix):
    [head, tail] = os.path.split(fname)
    if head:
        os.chdir(head)

    contents = getFileContents(fname)
    nonempty = stripComments(contents)
    lineinfo = addFileNameAndNumber(nonempty)

    return getSectionLabels(lineinfo, label_prefix=label_prefix)


if __name__ == "__main__":
    if len(sys.argv) > 2:
        prefix = sys.argv[2]
    else:
        prefix = ''

    print(main(sys.argv[1], prefix))



# Author: Srinath Avadhanula
# This file is distributed as part of the vim-latex project
# http://vim-latex.sf.net

import re
import os

try:
    from urllib.request import urlopen, pathname2url
except ImportError:
    from urllib import urlopen, pathname2url

# Compatibility functions
# Check for existence of builtin function next()
try:
    next
except NameError:
    def next(it):
        return it.next()

# Define items(dict) as an iterator over the items
if not("iteritems" in dir(dict())):
    # In python3, the job of iteritems() is done by items()
    def items(dictionary):
        return dictionary.items()
else:
    # In python2, we use iteritems()
    def items(dictionary):
        return dictionary.iteritems()


class Bibliography(dict):
    def __init__(self, txt, macros={}):
        """
        txt:
            a string which represents the entire bibtex entry. A typical
            entry is of the form:
                @ARTICLE{ellington:84:part3,
                  author = {Ellington, C P},
                  title = {The Aerodynamics of Hovering Insect Flight. III. Kinematics},
                  journal = {Philosophical Transactions of the Royal Society of London. Series B, Biological Sciences},
                  year = {1984},
                  volume = {305},
                  pages = {41-78},
                  number = {1122},
                  owner = {Srinath},
                  pdf = {C:\srinath\research\papers\Ellington-3-Kinematics.pdf},
                  timestamp = {2006.01.02},
                }
        """

        for k, v in items(macros):
            txt = txt.replace(k, '{' + v + '}')

        m = re.match(r'\s*@(\w+)\s*{\s*((\S+),)?(.*)}\s*', txt,
                     re.MULTILINE | re.DOTALL)
        if not m:
            return None

        self['bibtype'] = m.group(1).capitalize()
        self['key'] = m.group(3)
        self['body'] = m.group(4)

        body = self['body']
        self['bodytext'] = ''
        while 1:
            m = re.search(r'(\S+?)\s*=\s*(.)', body)
            if not m:
                break

            field = m.group(1)

            body = body[(m.start(2) + 1):]
            if m.group(2) == '{':
                # search for the next closing brace. This is not simply a
                # matter of searching for the next closing brace since
                # braces can be nested. The following code basically goes
                # to the next } which has not already been closed by a
                # following {.
                mniter = re.finditer(r'{|}', body)

                count = 1
                while 1:
                    try:
                        mn = next(mniter)
                    except StopIteration:
                        return None

                    if mn.group(0) == '{':
                        count += 1
                    else:
                        count -= 1

                    if count == 0:
                        value = body[:(mn.start(0))]
                        break

            elif m.group(2) == '"':
                # search for the next unquoted double-quote. To be more
                # precise, a double quote which is preceded by an even
                # number of double quotes.
                mn = re.search(r'(?!\\)(\\\\)*"', body)
                if not mn:
                    return None

                value = body[:(mn.start(0))]

            else:
                # $ always matches. So we do not need to do any
                # error-checking.
                mn = re.search(r',|$', body)
                value = m.group(2) + body[:(mn.start(0))].rstrip()

            self[field.lower()] = re.sub(r'\s+', ' ', value)
            body = body[(mn.start(0) + 1):]

            self['bodytext'] += ('  %s: %s\n' % (field, value))
            if self['bibtype'].lower() == 'string':
                self['macro'] = {field: value}

        self['bodytext'] = self['bodytext'].rstrip()

    def __getitem__(self, key):
        try:
            return dict.__getitem__(self, key)
        except KeyError:
            return ''

    def __str__(self):
        if self['bibtype'].lower() == 'string':
            return 'String: %(macro)s' % self

        elif self['bibtype'].lower() == 'article':
            return ('Article [%(key)s]\n' +
                    'TI "%(title)s"\n' +
                    'AU %(author)s\n' +
                    'IN In %(journal)s, %(year)s') % self

        elif self['bibtype'].lower() == 'conference':
            return ('Conference [%(key)s]\n' +
                    'TI "%(title)s"\n' +
                    'AU %(author)s\n' +
                    'IN In %(booktitle)s, %(year)s') % self

        elif self['bibtype'].lower() == 'mastersthesis':
            return ('Masters [%(key)s]\n' +
                    'TI "%(title)s"\n' +
                    'AU %(author)s\n' +
                    'IN In %(school)s, %(year)s') % self

        elif self['bibtype'].lower() == 'phdthesis':
            return ('PhD [%(key)s]\n' +
                    'TI "%(title)s"\n' +
                    'AU %(author)s\n' +
                    'IN In %(school)s, %(year)s') % self

        elif self['bibtype'].lower() == 'book':
            return ('Book [%(key)s]\n' +
                    'TI "%(title)s"\n' +
                    'AU %(author)s\n' +
                    'IN %(publisher)s, %(year)s') % self

        else:
            s = '%(bibtype)s [%(key)s]\n' % self
            if self['title']:
                s += 'TI "%(title)s"\n' % self
            if self['author']:
                s += 'AU %(author)s\n' % self

            for k, v in items(self):
                if k not in ['title', 'author', 'bibtype', 'key', 'id', 'file',
                             'body', 'bodytext']:
                    s += 'MI %s: %s\n' % (k, v)

            return s.rstrip()

    def satisfies(self, filters):
        for field, regexp in filters:
            if not re.search(regexp, self[field], re.I):
                return False

        return True


class BibFile:

    def __init__(self, filelist=''):
        self.bibentries = []
        self.filters = []
        self.macros = {}
        self.sortfields = []
        if filelist:
            for f in filelist.splitlines():
                self.addfile(f)

    def addfile(self, file):
        fields = urlopen('file://' + pathname2url(os.path.abspath(file))).read().decode('utf-8').split('@')
        for f in fields:
            if not (f and re.match('string', f, re.I)):
                continue

            b = Bibliography('@' + f)
            self.macros.update(b['macro'])

        for f in fields:
            if not f or re.match('string', f, re.I):
                continue

            b = Bibliography('@' + f, self.macros)
            if b:
                b['file'] = file
                b['id'] = len(self.bibentries)
                self.bibentries += [b]

    def addfilter(self, filterspec):
        self.filters += [filterspec.split()]

    def rmfilters(self):
        self.filters = []

    def __str__(self):
        s = ''
        for b in self.bibentries:
            if b['key'] and b.satisfies(self.filters):
                s += '%s\n\n' % b
        return s

    def addsortfield(self, field):
        self.sortfields += [field]

    def rmsortfields(self):
        self.sortfields = []

    def sort(self):
        self.bibentries.sort(key=lambda x:[x[field] for field in self.sortfields])

if __name__ == "__main__":
    import sys

    bf = BibFile(sys.argv[1])
    print(bf)
